#ifndef LTL_INS_H
#define LTL_INS_H

#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * LITTLE Bytecode instructions are 64bit wide, using
 * 16 bits for opcode and 24 bits each for operand 1 and 2.
 */
enum ltl_instructionset {

	LTL_INS_EOC	=  0,	/* End Of Code pseudo instruction		*/

	LTL_INS_C4OBJ	=  1,	/* Class For OBJect				*/
	LTL_INS_C4SYM	=  2,	/* Class For SYMbol				*/
	LTL_INS_C4SELF	=  3,	/* Class For SELF				*/
	LTL_INS_SLD	=  4,	/* Self LoaD					*/
	LTL_INS_SALD	=  5,	/* Self/Attribute index LoaD			*/
	LTL_INS_SAST	=  6,	/* Self/Attribute index STore			*/
	LTL_INS_OMCL	=  7,	/* Object Method CaLl				*/
	LTL_INS_OTMCL	=  8,	/* Object Tid x Mid CaLl			*/
	LTL_INS_SMCL	=  9,	/* Self Method CaLl				*/
	LTL_INS_STMCL	= 10,	/* Self Tid x Mid CaLl				*/
	LTL_INS_LLD	= 11,	/* Local LoaD					*/
	LTL_INS_LST	= 12,	/* Local STore					*/
	LTL_INS_DUP	= 13,	/* DUPlicate item on top of execution stack	*/
	LTL_INS_EXCH	= 14,	/* EXCHange 2 topmost items on execution stack	*/
	LTL_INS_POP	= 15,	/* POP execution stack top			*/
	LTL_INS_POPN	= 16,	/* POP execution stack top			*/
	LTL_INS_JZ	= 17,	/* Jump if Zero					*/
	LTL_INS_J	= 18,	/* Jump						*/
	LTL_INS_RET	= 19,	/* RETurn					*/
	LTL_INS_XENTER	= 20,	/* eXception frame ENTER			*/
	LTL_INS_XEXIT	= 21,	/* eXception frame EXIT				*/
	LTL_INS_CNIL	= 22,	/* Create NIL					*/
	LTL_INS_CPSZ	= 23,	/* Create Pointer to char			*/
	LTL_INS_CSYM	= 24,	/* Create SYMbol				*/
	LTL_INS_CWORD	= 25	/* Create INT					*/
};

typedef struct ltl_ins {

#if LTL_LITTLE_ENDIAN
	unsigned ins:16;
	unsigned flags:16;
#else
	unsigned flags:16;
	unsigned ins:16;
#endif
	unsigned arg1, arg2, arg3: 32;

} ltl_ins_t;

/*
 * Returns a memnonic representation for the instruction.
 */
char *ltl_get_memnonic(ltl_ins_t);
/*
 * Prints an instruction to a file.
 */
void ltl_print_instruction(FILE *,ltl_ins_t);
/*
 * Prints a block of code to a file.
 * The block must be terminated by the EOC pseudo instruction.
 */
void ltl_print_codeblock(FILE *,ltl_ins_t *);

#ifdef __cplusplus
}
#endif

#endif
